#ifndef _BCTYPE_HH
#define _BCTYPE_HH _BCTYPE_HH
#include<dune/pdelab/common/function.hh>

/* *************************************************** *
 * 
 *   Class representing boundary conditions
 *
 * *************************************************** */
template<typename GV>
class BCType : public Dune::PDELab::BoundaryGridFunctionBase<
        Dune::PDELab::BoundaryGridFunctionTraits<GV,int,1,
        Dune::FieldVector<int,1> >,BCType<GV> > {

  public:
    typedef Dune::PDELab::template BoundaryGridFunctionTraits<
        GV,int,1,Dune::template FieldVector<int,1> > Traits;

/* *************************************************** *
 * Constructor
 * *************************************************** */
    BCType (const GV& gv_) : gv(gv_) {}
    
/* *************************************************** *
 * Evaluate
 * *************************************************** */
    template<typename I>
    inline void evaluate(I& i,
                         const typename Traits::DomainType& xlocal,
                         typename Traits::RangeType& y) const {
      Dune::FieldVector<typename GV::Grid::ctype, GV::dimension> x = i.geometry().global(xlocal);
      // Default = Dirichlet
      y = 1;
      // Neumann at bottom boundary
#ifdef USE_ALUGRID
      if (abs(x[0]*x[0]+x[1]*x[1]+x[2]*x[2]-1.) < 1.0e-15)
        y = 0;
#else
      if (x[2] < 1.0e-10)
        y = 0;
#endif
    }

/* *************************************************** *
 * Return gridview
 * *************************************************** */
    inline const GV& getGridView() { return gv; }

  private:
    const GV& gv;
}; 

#endif
